patches-own [ ;a patch is a cell

  w; concentration of species w

  x; concentration of species x

  y; concentration of species y

  z; concentration of species z

  dw_reaction; variation in concentration of species w due to the reaction

  dx_reaction; variation in concentration of species x due to the reaction

  dy_reaction; variation in concentration of species y due to the reaction

  dz_reaction; variation in concentration of species z due to the reaction

  dw_diffusion; variation in concentration of species w due to the reaction

  dx_diffusion; variation in concentration of species x due to the reaction

  dy_diffusion; variation in concentration of species y due to the reaction

  dz_diffusion; variation in concentration of species z due to the reaction

  p_signal ; signal intensity


]

globals [


  w1 w2; internal boundaries used for w initialization

  x1 x2; interval boundaries used for x initialization

  y1 y2; interval boundaries used for y initialization

  z1 z2; internal boundaries used for z initialization

  maxi

  t

  plate-sizex
  plate-sizey


]

to Setup-random ; Random-setup button initialization
  clear-all

  set plate-sizex round (10 * dimx * max-pxcor) ; size of x dimension of the plate (world)
  set plate-sizey round (10 * dimy * max-pycor) ; size of x dimension of the plate (world)
  set t t0
  set-patch-size 4 ; size of a patch

  ask patches [
    ifelse Manual_concentrations? ; Manual concentrations if activated
    [
      set w w_0
      set x x_0
      set y y_0
      set z z_0]
    [ set w random-float 2
      set x random-float 2
      set y random-float 2
      set z random-float 2]

    ifelse Equidistant_signal? [ ; Signal only appearing in some equidistant patches if activated
    ask patches with [pxcor mod gap = 1 and pycor mod gap = 1] [
         set p_signal p]
    ]
        [set p_signal p]

    draw-plate
    ]

  reset-ticks
end

to-report act [F K_act nn] ; activation function used in reac
  report ((F / K_act) ^ nn)/(1 + ((F / K_act) ^ nn))
end

to-report rep [F K_rep nn] ; repression function used in reac
  report (1 / (1 + (F / K_rep) ^ nn))
end

to go ; Go button, application of all procedures

  ask patches

    [
      reac

      diff

      update

      draw-plate

  ]

  set t t + 1

  tick

end


to reac ; diffusion procedure

  if ((abs pycor) <= plate-sizey) and ((abs pxcor) <= plate-sizex)

    [reaction]

end

to diff ; diffusion procedure

  if ((abs pycor) <= plate-sizey) and ((abs pxcor) <= plate-sizex)

    [diffusion]

end


to-report S [temps] ; signal definition
  ifelse p_signal = 0
  [report 0]
  [
  ifelse (Smax * sin(2 * pi / p_signal * (temps + t0)) ^ 1) >= 0 ; t0 is signal shift
  [report (Smax * sin(2 * pi / p_signal * (temps + t0)) ^ 1)]
  [report 0]
  ]
end



to reaction ; calculates deltareaction for each patch

 set   dw_reaction  (k * (act (S t) KSa nSa) * (rep x Krep n) * (rep y Krep1 n) - kd * w) * dt    ;  kd (Krep) is the degradation (repression) factor of w and y species
 set   dx_reaction  (k * (rep (S t) KSr nSr) * (rep y Krep1 n) * (rep z Krep1 n) - kd1 * x) * dt ; kd1 (Krep1) is the degradation (repression) factor of x and z species
 set   dy_reaction  (k * (act (S t) KSa nSa) * (rep z Krep n) * (rep w Krep1 n) - kd * y) * dt    ;  kd (Krep) is the degradation (repression) factor of w and y species
 set   dz_reaction  (k * (rep (S t) KSr nSr) * (rep w Krep1 n) * (rep x Krep1 n) - kd1 * z) * dt ; kd1 (Krep1) is the degradation (repression) factor of x and z species


end

to diffusion ; calculates deltadiffusion for each patch

set dw_diffusion (Diffw * Diffall * ((sum [ w ] of neighbors) / (count neighbors) - w))

set dx_diffusion (Diffx * Diffall * ((sum [ x ] of neighbors) / (count neighbors) - x))

set dy_diffusion (Diffy * Diffall * ((sum [ y ] of neighbors) / (count neighbors) - y))

set dz_diffusion (Diffz * Diffall * ((sum [ z ] of neighbors) / (count neighbors) - z))

end



to update; updates w, x, y, z via reaction and diffusion

  set w (w + dw_reaction + dw_diffusion)

  set x (x + dx_reaction + dx_diffusion)

  set y (y + dy_reaction + dy_diffusion)

  set z (z + dz_reaction + dz_diffusion)

draw-plate

end


to draw-plate  ; repaint all patches within defined plate dimensions

  if ((abs pycor) <= plate-sizey) and ((abs pxcor) <= plate-sizex)
    [repaint]

end



to repaint ; defines the color of the patch that corresponds to its state ( up on concentration of x, y, z and w )

  ifelse w > x + y + z   ; w is dominant, intense red color
  [ set pcolor 15]
  [ ifelse x > w + y + z  ; x is dominant, intense yellow color
    [ set pcolor 45 ]
    [ ifelse y > w + x + z ; y is dominant, intense green color
      [set pcolor 55]
      [ifelse z > w + x + y ; z is dominant, intense blue color
        [set pcolor 95]
        [set maxi max(list w x y z)
          ifelse (maxi = w) and (w > 0.4 * (w + x + y + z)) ; w is dominant, above 40% of the patch content, red color
          [set pcolor 16]
          [ifelse (maxi = w) and (w > 0.3 * (w + x + y + z)); w is dominant, above 30% of the patch content, light red color
            [set pcolor 17]
            [ifelse (maxi = w) and (w > 0.25 * (w + x + y + z)); w is dominant, above 25% of the patch content, very light red color
              [set pcolor 18]
              [ifelse (maxi = x) and (x > 0.4 * (w + x + y + z))
                [set pcolor 46]
                [ifelse (maxi = x) and (x > 0.3 * (w + x + y + z))
                  [set pcolor 47]
                  [ifelse (maxi = x) and (x > 0.25 * (w + x + y + z))
                    [set pcolor 48]
                    [ifelse (maxi = y) and (y > 0.4 * (w + x + y + z))
                      [set pcolor 56]
                      [ifelse (maxi = y) and (y > 0.3 * (w + x + y + z))
                        [set pcolor 57]
                        [ifelse (maxi = y) and (y > 0.25 * (w + x + y + z))
                          [set pcolor 58]
                          [ifelse (maxi = z) and (z > 0.4 * (w + x + y + z))
                            [set pcolor 96]
                            [ifelse (maxi = z) and (z > 0.3 * (w + x + y + z))
                              [set pcolor 97]
                              [ifelse (maxi = z) and (z > 0.25 * (w + x + y + z))
                                [set pcolor 98]
                                [set pcolor 0]

                              ]
                            ]
                          ]
                        ]
                      ]
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      ]
    ]
  ]

end
@#$#@#$#@
GRAPHICS-WINDOW
489
21
821
354
-1
-1
4.0
1
10
1
1
1
0
1
1
1
-40
40
-40
40
0
0
1
ticks
30.0

BUTTON
408
37
471
70
Go
go\n
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
1272
319
1805
500
Concentration evolution
Time step
Concentration
0.0
2000.0
0.0
200000.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "plot sum [w] of patches"
"pen-1" 1.0 0 -1184463 true "" "plot sum [x] of patches"
"pen-2" 1.0 0 -10899396 true "" "plot sum [y] of patches"
"pen-3" 1.0 0 -11221820 true "" "plot sum [z] of patches"
"pen-4" 1.0 0 -6459832 true "" ""

SLIDER
1362
535
1534
568
dimx
dimx
0
10
10.0
0.1
1
NIL
HORIZONTAL

SLIDER
1559
535
1731
568
dimy
dimy
0
10
10.0
0.1
1
NIL
HORIZONTAL

BUTTON
19
33
152
66
NIL
Setup-random
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
18
154
190
187
k
k
0
3000
3000.0
10
1
NIL
HORIZONTAL

SLIDER
207
115
379
148
Smax
Smax
0
300
300.0
1
1
NIL
HORIZONTAL

SLIDER
207
76
379
109
t0
t0
0
100
0.0
1
1
NIL
HORIZONTAL

SLIDER
18
116
190
149
KSa
KSa
0.1
100
0.1
0.1
1
NIL
HORIZONTAL

SLIDER
18
306
190
339
nSa
nSa
0
100
4.0
1
1
NIL
HORIZONTAL

SLIDER
207
191
379
224
n
n
0
10
4.0
1
1
NIL
HORIZONTAL

SLIDER
18
192
190
225
kd
kd
0
200
10.0
1
1
NIL
HORIZONTAL

SLIDER
18
76
190
109
KSr
KSr
0.1
100
0.1
0.1
1
NIL
HORIZONTAL

SLIDER
18
267
190
300
nSr
nSr
0
100
4.0
1
1
NIL
HORIZONTAL

SLIDER
206
154
378
187
p
p
0.01
100
1.0
0.01
1
NIL
HORIZONTAL

SLIDER
207
307
379
340
dt
dt
0
0.1
0.001
0.001
1
NIL
HORIZONTAL

SLIDER
1292
10
1464
43
Diffw
Diffw
0
1
0.01
0.01
1
NIL
HORIZONTAL

SLIDER
1292
51
1464
84
Diffx
Diffx
0
1
0.01
0.01
1
NIL
HORIZONTAL

SLIDER
1293
90
1465
123
Diffy
Diffy
0
1
0.01
0.01
1
NIL
HORIZONTAL

SLIDER
1293
132
1465
165
Diffz
Diffz
0
1
0.01
0.01
1
NIL
HORIZONTAL

SWITCH
1496
12
1728
45
Manual_concentrations?
Manual_concentrations?
1
1
-1000

SLIDER
1496
51
1668
84
w_0
w_0
0
50
0.0
1
1
NIL
HORIZONTAL

SLIDER
1498
89
1670
122
x_0
x_0
0
50
0.0
1
1
NIL
HORIZONTAL

SLIDER
1499
125
1671
158
y_0
y_0
0
50
0.0
1
1
NIL
HORIZONTAL

SLIDER
1500
163
1672
196
z_0
z_0
0
50
0.0
1
1
NIL
HORIZONTAL

PLOT
1272
189
1472
309
Signal
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot sum [S t] of patches"

SWITCH
1501
202
1702
235
Equidistant_signal?
Equidistant_signal?
1
1
-1000

SLIDER
1502
244
1674
277
gap
gap
0
50
2.0
1
1
NIL
HORIZONTAL

SLIDER
207
230
379
263
Krep
Krep
1
100
30.0
1
1
NIL
HORIZONTAL

SLIDER
207
269
380
302
Krep1
Krep1
0
100
30.0
1
1
NIL
HORIZONTAL

SLIDER
19
230
192
263
kd1
kd1
0
200
10.0
10
1
NIL
HORIZONTAL

SLIDER
1105
73
1278
106
Diffall
Diffall
0
10
10.0
0.5
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
